function [Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch, sp)
%MAKEYBUS   Builds the bus admittance matrix and branch admittance matrices.
%   [Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch, sp) returns the full
%   bus admittance matrix (i.e. for all buses) and the matrices Yf and Yt
%   which, when multiplied by a complex voltage vector, yield the vector
%   currents injected into each line from the "from" and "to" buses
%   respectively of each line. If sp is 0 it does not use sparse matrices,
%   otherwise it does. Does appropriate conversions to p.u.

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%% create sparse matrices by default
if nargin < 4
	sp = 1;
end

%% constants
j = sqrt(-1);
nb = size(bus, 1);			%% number of buses
nl = size(branch, 1);		%% number of lines

%% define named indices into bus, branch matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;

%% check that bus numbers are equal to indices to bus (one set of bus numbers)
if any(bus(:, BUS_I) ~= [1:nb]')
	error('buses must appear in order by bus number')
end

%% for each branch, compute the elements of the branch admittance matrix where
%%
%%		| If |   | Yff  Yft |   | Vf |;
%%		|    | = |          | * |    |;
%%		| It |   | Ytf  Ytt |   | Vt |;
%%
f = branch(:, F_BUS);							%% list of "from" buses
t = branch(:, T_BUS);							%% list of "to" buses
stat = branch(:, BR_STATUS);					%% ones at in-service branches
Ys = stat ./ (branch(:, BR_R) + j * branch(:, BR_X));	%% series admittance
Bc = stat .* branch(:, BR_B);							%% line charging susceptance
tap = ones(nl, 1);								%% default tap ration = 1
i = find(branch(:, TAP));						%% indices of non-zero tap ratios
tap(i) = branch(i, TAP);						%% assign non-zero tap ratios
tap = tap .* exp(j*pi/180 * branch(:, SHIFT));	%% add phase shifters
Ytt = Ys + j*Bc/2;
Yff = Ytt ./ (tap .* conj(tap));
Yft = - Ys ./ conj(tap);
Ytf = - Ys ./ tap;

%% Build Yf and Yt such that Yf * V is the vector of complex branch currents injected
%% at each branch's "from" bus, and Yt is the same for the "to" bus end
if nargout > 1
	i = [[1:nl]'; [1:nl]'];		%% double set of row indices	
	if sp						%% sparse version
		Yf = sparse(i, [f; t], [Yff; Yft]);
		Yt = sparse(i, [f; t], [Ytf; Ytt]);
	else						%% dense version
		Yf = zeros(nl, nb);
		Yt = zeros(nl, nb);
		Yf(i, [f; t]) = [Yff; Yft];
		Yt(i, [f; t]) = [Ytf; Ytt];
	end
end

%% initialize admittance matrix Ybus with shunt data on diagonal

%% if Ps is the real power consumed by the shunt at V = 1.0 p.u.
%% and Qs is the reactive power injected by the shunt at V = 1.0 p.u.
%% then Ps - j Qs = V * conj(Ys * V) = conj(Ys) = Gs - j Bs,
%% i.e. Ys = Ps + j Qs, so ...
Ys = (bus(:, GS) + j * bus(:, BS)) / baseMVA;	%% vector of shunt admittances
if sp							%% sparse version
	Ybus = spdiags(Ys, 0, nb, nb);
else							%% dense version
	Ybus = diag(Ys);
end

%% add branch data to admittance matrix Ybus
%% note: can't vectorize easily since we might have parallel lines
for i = 1:nl
	Ybus( f(i), f(i) ) = Ybus( f(i), f(i) ) + Yff(i);
	Ybus( f(i), t(i) ) = Ybus( f(i), t(i) ) + Yft(i);
	Ybus( t(i), f(i) ) = Ybus( t(i), f(i) ) + Ytf(i);
	Ybus( t(i), t(i) ) = Ybus( t(i), t(i) ) + Ytt(i);
end

return;
