function [f, g] = OTSfun(x, baseMVA, bus, gen, gencost, branch, Ybus, Yf, Yt, V, ref, pv, pq, alg)
%OTSFUN  Evaluates objective function & constraints for OPF.
%   [f, g] = OTSfun(x, baseMVA, bus, gen, gencost, branch, Ybus, Yf, Yt, V, ref, pv, pq, alg)

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%%----- initialize -----
vVg = varVg(alg);

%% constant
j = sqrt(-1);

%% define named indices into gen, branch matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;
[PW_LINEAR, POLYNOMIAL, MODEL, STARTUP, SHUTDOWN, N, COST] = idx_cost;

%% set up indexing for x
nb = size(bus, 1);
nl = size(branch, 1);
npv	= length(pv);
npq	= length(pq);
ng = npv + 1;
j1 = 1;			j2 = npv;			%% j1:j2 - V angle of pv buses
j3 = j2 + 1;	j4 = j2 + npq;		%% j3:j4 - V angle of pq buses
if vVg
	j5 = j4 + 1;	j6 = j4 + nb;	%% j5:j6 - V mag of all buses
else
	j5 = j4 + 1;	j6 = j4 + npq;	%% j5:j6 - V mag of pq buses
end
j7 = j6 + 1;	j8 = j6 + ng;		%% j7:j8 - P of generators

%% grab Pg
Pg = x(j7:j8);								%% power generation in p.u.

%%----- evaluate objective function -----
%% generator info
on = find(gen(:, GEN_STATUS));				%% which generators are on?
gbus = gen(on, GEN_BUS);					%% what buses are they at?

%% put Pg back in gen
gen(on, PG) = Pg * baseMVA;					%% real generation in MW

%% compute objective value
f = sum(totcost(gencost(on, :), gen(on, PG)));

%%----- evaluate constraints -----
if nargout > 1
	%% reconstruct V
	if vVg
		Va = zeros(nb, 1);
		Va(ref) = angle(V(ref));
		Vm = x(j5:j6);
	else
		Va = angle(V);	Vm = abs(V);
		Vm(pq) = x(j5:j6);
	end
	Va(pv) = x(j1:j2);
	Va(pq) = x(j3:j4);
	V = Vm .* exp(j * Va);
	
	%% rebuild Sbus
	Sbus = makeSbus(baseMVA, bus, gen);		%% net injected power in p.u.
	
	%% evaluate power flow equations
	mis = V .* conj(Ybus * V) - Sbus;
	
	%% compute branch power flows
	Sf = V(branch(:, F_BUS)) .* conj(Yf * V);	%% complex power at "from" bus (p.u.)
	St = V(branch(:, T_BUS)) .* conj(Yt * V);	%% complex power injected at "to" bus (p.u.)
	
	%% compute generator reactive power in p.u.
	%% Q needed by grid + Q needed by load
	%% = Q mismatch + Q (used to compute mismatch) from gen
	Qg = imag(mis(gbus)) + gen(on, QG) / baseMVA;
	
	%% compute constraint function values
	%% voltage constraints
	if vVg
		Vmconstraint = [
			bus(:, VMIN) - x(j5:j6);		%% lower voltage limit for all buses
			x(j5:j6) - bus(:, VMAX);		%% upper voltage limit for all buses
		];
	else
		Vmconstraint = [
			bus(pq, VMIN) - x(j5:j6);		%% lower voltage limit for pq buses
			x(j5:j6) - bus(pq, VMAX);		%% upper voltage limit for pq buses
		];
	end
	g = [
		%% equality constraints
		real(mis(gbus));					%% real power mismatch for ref & pv buses
		real(mis(pq));						%% real power mismatch for pq buses
		imag(mis(pq));						%% reactive power mismatch for pq buses

		%% inequality constraints (variable limits, voltage & real generation)
		Vmconstraint;						%% lower/upper voltage limits
		gen(on, PMIN) / baseMVA - x(j7:j8);	%% lower generator P limit
		x(j7:j8) - gen(on, PMAX) / baseMVA;	%% upper generator P limit

		%% inequality constraints (reactive generation & line flow limits)
		gen(on, QMIN) / baseMVA - Qg;		%% lower generator Q limit
		Qg - gen(on, QMAX) / baseMVA;		%% upper generator Q limit
% 		real(Sf) - branch(:, RATE_A) / baseMVA;	%% branch real power limits (from bus)
% 		real(St) - branch(:, RATE_A) / baseMVA;	%% branch real power limits (to bus)
		abs(Sf) - branch(:, RATE_A) / baseMVA;	%% branch apparent power limits (from bus)
		abs(St) - branch(:, RATE_A) / baseMVA;	%% branch apparent power limits (to bus)
	];
end

return;
