function [bus, gen, branch, f, success, et] = uopf(baseMVA, bus, gen, gencost, ...
					branch, Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose)
%UOPF  Solves combined unit commitment / optimal power flow.
%   [bus, gen, branch, f, success, et] = uopf(baseMVA, bus, gen, gencost, ...
%                   branch, Ybus, Yf, Yt, V0, Pg0, ref, pv, pq, alg, verbose)
%   Assumes that Ybus, Yf, Yt, V, ref, pv, pq are consistent with
%   (or override) data in bus, gen, branch. If verbose true prints progress
%   info, if verbose > 1 it prints the output of each individual opf.

%   MATPOWER Version 1.0.1
%   by Ray Zimmerman, PSERC Cornell    9/19/97
%   Copyright (c) 1996, 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

%%----- initialization -----
%% flag indicating that unit commitment is not yet complete
more_to_commit = 1;

%% initialize arguments
if nargin < 15
	verbose = 0;		%% default: don't print stuff
	if nargin < 14
		alg = 0;		%% use default algorithm for this type of cost model
	end
end

%% define named indices into bus, gen, branch matrices
[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[PW_LINEAR, POLYNOMIAL, MODEL, STARTUP, SHUTDOWN, N, COST] = idx_cost;

%% save actual minimum generator real power output limits
Pmin0 = gen(:, PMIN);

%% initialize flags indicating machines for which commitment is not yet finalized
undecided = ones(size(gen, 1));		%% start with all undecided
i = find(Pmin0 == 0);				%% find gens with Pmin == 0
undecided(i) = zeros(size(i));		%% mark as decided (prevents becoming PQ if set to zero)
i = find(~gen(:, GEN_STATUS));		%% find unavailable generators
undecided(i) = zeros(size(i));		%% mark as decided
on0 = find(gen(:, GEN_STATUS));		%% which generators are available?

%% set up initial state
last_shutdown = [];			%% no machines have been shut down yet
on = on0;					%% all available gens initially on
Pmin = zeros(size(Pmin0));	%% initialize all Pmin's to zero
V = V0;
Pg = gen(:, PG);
Pg(on) = Pg0 * baseMVA;

%%-----  do combined unit commitment/optimal power flow  -----
t0 = clock;									%% start timer
while more_to_commit
	%% copy stuff to gen
	gen(:,  GEN_STATUS) = zeros(size(gen, 1), 1);	%% all off ...
	gen(on, GEN_STATUS) =  ones(length(on),  1);	%% ... except those turned on
	gen(:, PMIN) = Pmin;

	%% reassign bus types
	[ref, pv, pq] = bustypes(bus, gen);

	%% run opf
	[bus, gen, branch, f, success, et] =  opf(baseMVA, bus, gen, gencost, branch, ...
					Ybus, Yf, Yt, V, Pg(on) / baseMVA, ref, pv, pq, alg, verbose);
	Pg = gen(:, PG);
	if success
		if verbose > 1
			printopf(bus, gen, branch, f, success, et, 1, 1);
			fprintf('\nHit any key to continue ...');
			pause;
			fprintf('\n');
		end

		%% need to change commitment?
		shutdown = [];					%% initialize shutdown list
		%% find undecided & Pg == 0
		ii = find(undecided(on) & Pg(on) < eps);
		%% find undecided & 0 < Pg < Pmin
		jj = find(undecided(on) & Pg(on) > eps & Pg(on) < Pmin0(on));
		if any(ii) & success		%% any undecided units dispatched at 0?
			%% remove one with largest mu_Pmin from commitment schedule
			[junk, i] = max(gen(on(ii), MU_PMIN));
			i = ii(i(1));				%% convert to index to on
			shutdown = [shutdown; i];	%% put on shutdown list
		elseif any(jj) & success	%% any undecided units dispatched between 0 and Pmin?
			%% decide commitment of infeasible generator "closest" to feasibility
			nPg = Pg(on(jj)) ./ Pmin0(on(jj)) - ones(size(jj)) / 2; %% normalize & shift
			[junk, i] = max(abs(nPg));	%% pick one closest to feasibility
			nPg = nPg(i(1));
			i = jj(i(1));				%% index to on
	
			%% force it to be feasible
			if nPg > 0					%% include in commitment
				Pmin(on(i)) = Pmin0(on(i));		%% set Pmin to actual
				undecided(on(i)) = 0;			%% mark as decided
				if verbose
					fprintf('Pmin enforced at generator %d\n', on(i));
				end
			else						%% exclude from commitment
				shutdown = [shutdown; i];		%% put on shutdown list
			end
		else							%% hey, we're done!
			more_to_commit = 0;
		end
	
		%% shut down selected units
		for k = 1:length(shutdown)
			i = shutdown(k);			%% which gen?
			if verbose
				fprintf('Shutting down generator %d\n', on(i));
			end
			
			%% set generation to zero
			gen(on(i), PG) = 0;
			gen(on(i), QG) = 0;
			
			%% update Pg, on, undecided
			last_shutdown = on(i);
			Pg(on(i)) = 0;
			undecided(on(i)) = 0;					%% mark as decided
			on = on([1:(i-1) (i+1):length(on)]');	%% turn it off
		end
	else	%% opf failed
		if last_shutdown
			%% try firing up last machine that was shut down, better than shedding load
			on = sort([on; last_shutdown]);
			i = find(on == last_shutdown);
			Pmin(on(i)) = Pmin0(on(i));		%% set Pmin to actual
			gen(on(i), PG) = Pmin(on(i));	%% set generation to Pmin
			Pg(on(i)) = Pmin(on(i));
			last_shutdown = [];
			if verbose
				fprintf('OPF failed, firing up generator %d\n', on(i));
			end
		else
			%% nothing shutdown recently, bail out
			if verbose
				fprintf('UOPF failed\n');
			end
			return;
		end
	end
end

%% compute elapsed time
et = etime(clock, t0);

return;
